__code__='GatU0D/[on&>$QoP,!^fUm1U)cr<Nch?>;5TaJ1I+WreskRL.1J1SaI8PFN?g"f:R;YUA-?i$Dkf/g7H+TVlp\\D&%-GA$&`B%Aa]\'Yep^J\'^%POhaUD,Y@#OTR\\s$Y6"V?$;c=U[$\\K0<<8mm"fqaU!?6PZ!2fgp!LnU0!P\\[5(@W1!EPNI-mf6u$:B8Z@ILgZ9n-?a-J>WFo!1j?V#C%<Z!oQ27*.LgsXF&:>_6XIWCG;iBC+o`i"\'(-.<=gh`Ksdg9GY71_=Up;(72WC\\U_p0UYniYq<G:7NpuY*q(B[)q+heOBX?qE`@Z@TaU/oJjX"SNT"RHLUq5Oc;@%ToR]b:Fq"^<F"WfGMV"gU4d*[t)sq*F"\\R3V@J;3Lp1nQ5SuH]OK\\[2@%-+gJ"_@\'-k[>2Kj"UYN"$G$43p*iiP\\k"anF(M.\'Dp;h&R7T`Gi*3^+=$4.`RM:KGGfA#q)lkgadMQP1V7^;3:f@(Skfq("G13R9,fN^8PXI\'U5*0[jZ57\'g%pMJBmN)CY9r5oA[1f=59\')G6moZPuE//lCEp?,gk<p2"0.SI2BJp"Wh4W_\\ca?[aBW"Pk%At%q:<i-f@oX:jGDKe.R]=VqI2WJ2Y*N#U?bWY\\_e#)<V:(WHg)l8Bb[8`>@dc<,VZb=9E(m[j2\\Z\'4$_JZCH33soncWYIN4CIhBm1j]1(*<[nUU4I42GL#K"Dcm4XE`mUiID8gNr\\bG=3WVC-F!U5m_)NaUQf%-315bK\\.ok2cHTF:p)#rr/RD_YEV&IThelc8<?+.[Z\\Oo49WZZ6&EcHJrpH5IBG$7!r&l`to863G7Ma2&A_/^o>c(^Tgl@QB1?82(\\)d&$>:ArZ_;oiIpV&0WGIPLcj@0^"$4@)gRH=O(FM%>ASiFIe\'omEIpZ.MdNEp"!F5[2%QegX;&sEJOrtg8jf,8hp)?CRRh@Z\\,CiDmhfn;67\'%jeI@Mnjq(H$qEBRRYALJG`eq;dIW]"q,t*@:DA)qQ\'H/o+>Dk,tW):?;:<*"r?$>7D6dI4;I:A/q.W-[5EIqu#&H]aOc4]Udhda6%#9r?iN4@TkTPnNU<I\'9107hQ_N7~>'



__code__='Gat=(D/U(X&>$9MP6;5,8>3CeY#&;#WHNM8WiE\'K2%Ca7g^"-iR9$6/bpGGk^,u$7EMW(Q8H$tDeR-mq]fYQpWCZRTAoD$gT\')NtSHrhVfBN6f?MtBP]Eog5a!^ogh?)Hl)N4ei!QVttPJ9B6!9mhqC]M`]2?D,(c3#[!/cqUJ49tWni-E*GJ4kD7""j[^"C)!*"Rm&^A\\_(BY5j"Upc784^h\\I-!+%%;$a8iSp#s`&n-<A\\?o/+oaUD\\^j;]/k6+m19L+aM8P0cA-0:D*]38B$MlRS-,"drdE8r;?Sl2?lYhB[.Fd$%37#oj[`"H.RV(m)=]<b!&G?H1rt3I8:Wa6.ON3"g2p9c]0#H;s&M6r6)K;l`CQNj<B7;%<)OePa5O4Vf/<Qf\\@RDpORUDWtQhg!fq4n\'Tj$M\\5=o1-<p;`[K"4CLNs6)qF$u]5K/Lh/ogZe.`N6DG@G([%s"qN:i45;7i+%=I(c:2Aqe$M,E5UpY,8@G-m,fj\\X!d37/bYPpP).&"S\'=F"fcG;S4ED`KG1R_fhXG$ed]Z"DKi]6!tL7B:PYi\\9lMD?e[7`k?S.p<0k@)9Xo1:ZA8Vg_5[7ANT@U<%2!1:%gN3K:9F7ak*[uR\\E.T&cNddX^!KF38>]RbQRcMb))\'a9=gr0?raVjIE,a0)DD2C+g/0bGHU[#0Dqie8p;=h\'-I,LB;kBN?Z(d%OQ62\\,a0=XY-sV2D\'"aQ02E)A5%siB#2rjS)ea&MS_LI7-9;S&WaYHX`Ji3!:epVo*UnA@Z4b,m!L:up*UaDnm9c(4o9A+06o5^n+h,):<iP+0XkGc/,H=rNB&+4gps1uAjb,OQIq#p;eWKAYV>)m:bpTFI`52n<C:P"q(?Y2(C?MR`Wm.tm^g1XPWHZ?VqA\\=E&HOd75Kk_-:klT3"Y1]).Go+ER%c]rd);mGtLoU^4??u6N\\@;VY=V[DnmmW`m0!S8RRJG&bX)c+UC%;]LO&s3K`]$_[(fOEVXsZ*,NJfK4"blpP1Nfe3N>P0?DVMcY0j@k>43(rRlDqph!uLbO\'5oiZFQ97Yr@,sp2.)WX+:+0aL_X1JNL;i!Z&rP`-MEtc5>h6RJZ##66_\'SB<0X;Q)+B%u)^$ak(W:o=rm,Y^[N_0JIf]E0U6t~>'



__code__='Gas1]gMYJ*&Daab9/!XuKYE!gDQ8h<H@ZDHKRgEu885"n+io0bf=PnU3juZ4BiD\'\\8mhB*I`\\ek"]*@P#n8JJboTT&B$TbEk2Zabq-8^R\'"/2LfM..6+cJ0s7MtB*%ct&n*85ba25W+!)6CW2XMi7tN/b.M2T`,,.5oF_f_\'13H[;<cOK[1fRU\\;\\?K@=i][57Z3EU!Zcc5,LX0<OU`$qs+Mcd;\'D>U)0d>(ek"lN4C&P%J:;##u13hqd9&/:X/?Q\\>GK3cO1AScphcrkCh.rmsOFJe_R\\Ys1ZlZa/@SS3kNHYHjQ;[SKcYL-#kh##OUGlP4_\\[o)TC$Ac7LFZ.LOd5"C-mU!mP\\"B[jPb8OUJ1-t*CnkA^(X@ul9?j4nt$KNUEpl1Hqas_2E7^Z<=m;VG:O/kSK)9GP$f(=o*?qs8Q_t>[ZeY(9rLp4h8no$f?M<@[hH)cM"[Y1*d#*bg`sf^2(q&n@gMj%)R:TPBtA2rk=I4(_qQq*)BW-p+AL/j".r)H1G]WYmjg3N*f8WO.X>\'q*??0R,!QLL/_c=J[[GjMLL/X+<"MQ3/54nn!Wb-,S/hRNJ[HVLXr[tu:"6E,^E_!3j?FmW?L$U5$]4USEGPsloEp3<N*L>;FG4D5*_4A*XtMnY=.6OIf1`cmht88(SZ0U(KmEGG00FNm]_"%_>i`VY\'GllJV@S7gSmB]P]uH^b:RVAASagp7^E5j*_cKD[_rhn$k^Ak"Im4f?S(h6\\EgD^r:ZMD\\dc,^W"R=Y;(rTk>ZSh.6Y?EN.B#?>\'cEWu$3oYGfSJqRp9nX$r\\GE07AfX$o<VYCKYG(b%\\N^"Y.ZRPL.(O[)^"\\G2;,mNDb)<3\'Hlt$NP[A=#$OfNdT1YVh(AJ66(POAY5V8GPhPCh0MJfT*>?Jnd$VsYTKBfpZc=E3HP3[F&#ZNgjJ\\5,/:qS#/\\t@A;JI9Sarhbj[r;9f$0/-Zq`;^H2<@h%#CB%F:D2&t_0,3k[RK~>'



#copyright ReportLab Europe Limited. 2000-2016
#see license.txt for license details
#
##################################################################################
# This script assumes that McMillan's installer is in in c:\python\devel\Installer
#
#My DLLs are located as follows
#
# 1) reportlab\lib\pyHnj.pyd
#                 \_rl_accel.pyd
#                 \sgmlop.pyd
# 
# 2) rlextra\radxml\pyRXP.pyd
# 
# The original rml2pdf.py script is wrapped by rlextra\rml2pdf\exemain.py;
# the main intention of the wrapper is to allow for some primitive
# debugging and also stuff to force the path into a controlled state and
# therefore avoid clashing with existing python installations.
# 
# To make an exe try the following
# 
# cd \python\rlextra\rml2pdf
# 
# do_freeze.py -clean
# 
# if this runs OK to completion it should be able to make an exe and then
# copy it parallel to rml2pdf.py. Even so it doesn't follow that stuff
# works OK as the awful PIL location problem and or some other dlls may
# intervene and crash things.
##################################################################################
from __future__ import print_function
import sys, os, string

def list_print(name,L):
	print(name)
	for l in L:
		print('  ',l)

def items_print(name,L):
	for l, k in list(L.items()):
		print('  %s["%s"]="%s"' %(name,l,k))

def try_import(name):
	print('import',name, end=' ')
	try:
		exec('import '+name)
		print('succeeded')
		return 1
	except ImportError:
		print('failed')
		return 0

def getFlag(name,nextVal=0):
	r = name in sys.argv
	if r:
		x = sys.argv.index(name)
		del sys.argv[x]
		if nextVal:
			r = sys.argv[x]
			del sys.argv[x]
	return r

if __name__ == '__main__': #NO RUN TESTS
	try:
		import archive_rt
		exe = 1
		if getFlag('-rt_verbose'): archive_rt._verbose = 1
	except ImportError:
		exe = 0
	from reportlab import rl_config
	rl_config.defaultImageCaching = 0
	from reportlab.lib import pagesizes
	letter = getFlag('-letter')
	if letter:
		rl_config.defaultPageSize = pagesizes.letter
	from rlextra.rml2pdf import rml2pdf
	version = rml2pdf.version
	def Usage(msg=None,e=0):
		if msg:
			print(msg)
		print('Usage:\n  rml2pdf [options] rmlfiles....')
		print(' files rml files to be converted to pdf')
		print('    options')
		print('    -help     give this help')
		print('    -letter   paper size letter (default is A4)')
		print('    -path     print python path')
		print('    -env      print python environent')
		print('    -outdir   directory where output is stored, default')
		print('              is parallel to the input file')
		print('    -verbose  be verbose')
		print('    -import   show some python imports')
		print('    -version  show the rml2pdf version (%s)' % version)
		print('    -arcview  show the exe contents')
		print('    -arcviewx show the exe and pyz contents')
		print('    -timing   show the time used')
		sys.exit(e)

	if getFlag('-help'): Usage()

	showPath = getFlag('-path')
	showEnv = getFlag('-env')
	showImport = getFlag('-import')
	verbose = getFlag('-verbose')
	noconf = getFlag('-noconfirm')
	outDir = getFlag('-outdir',1)
	showVersion = getFlag('-version')
	arcView = getFlag('-arcview')
	if getFlag('-arcviewx'): arcView=2
	timing = getFlag('-timing')
		
	if not outDir: outDir = None
	if verbose: showPath = showEnv = showImport = 1
	if showPath: list_print('Initial sys.path',sys.path)
	if exe:
		exeDefinedPath = string.split(os.environ['PYTHONPATH'],';')
		newPath = []
		for p in sys.path:
			if p in exeDefinedPath: newPath.append(p)

		if verbose: print('Running as Exe')
		sys.path = newPath
		if showPath:
			list_print('Final sys.path', sys.path)
			list_print('sys.importers ', sys.importers)
		if arcView:
			try:
				from Installer.ArchiveViewer import printToc
				printToc(sys.argv[0],arcView==2)
			except:
				import traceback
				print("Can't print Archive Contents")
				traceback.print_exc()
	else:
		if verbose: print('Normal python')
	if showEnv: items_print('os.environ',os.environ)
	if showImport:
		try_import('PIL')
		try_import('reportlab.lib._rl_accel')
		try_import('rlextra.radxml.pyRXP')
	if verbose or showVersion:
		print('rml2pdf version %s' % version)
	FN=sys.argv[1:]
	for fn in FN:
		if not os.path.isfile(fn):
			Usage("'%s' is not a file" % fn, 1)

	if timing:
		import time
		t0 = time.time()
	rml2pdf.main(exe=exe,fn=FN,quiet=noconf,outDir=outDir)
	if timing: print('That took %.2f"' % (time.time()-t0))



__code__='Gas1]D/[on&>$:XQckN.oa"At-n^aMX+W.=\'NJ.5\'cdO:jBNo>89M;F*7#rdn_%NfQX$W].7jEc5LR<oTAd!P(]Di$k=i$Tjm4#TmERi?b]ocY)s2V$B`(7Cqhkm>J.]R1\'S$1%JaJR#,6:]X&b\'?-4%KI8?5(ZeX9$lQ(B>D)Mh"O;1Or,K*s%r:49sL?r"uXIGTLfAn,hN:^e93M!+#m-\'_!oo.tC\\%56nhj/E^?-bk:2//+>\'/>;3(.XE+To!a1.#\'?\'3$@,n\'e08\\Khr.7if;%D2df_Mfa)o6oQAlTsT.a\'$7X:\\GTAp(a$0J`ImY[]+J?A$%`"^A=\\@t7SIOFT\\!(Z1UQ!=n?O)2*%QqOg9b@[:/3!tGc2+\',OrRs?8E&3j#$nh].IbZfS.3dB5*o_;S0//G[jp9dr?68k=_;AI3_@hi#kUTHl$KfRGe8&f0=-`@4qoM7tGEiPCSqjp1$8iK[7GIH"lkV1/jPKb@>-Trl:2!)S0NW[ZCD:\'!,[Gdo1&MuU\'p;bVQMPfQr\':_HQ?h1k]\'0eP7;,)ou=m+!Q)t\'02cto#daF#rek%jZhR3kP<rhdBs4s/KeB)?`c"u=ILg8Q9Z5hjE3SDT@Xl1a$%rr0X,b/MHZFFn&ifRu8(F/Qe97h]6b8SfcsQI@ZiKG>cs_1.$\'OFf"jEq<MK7&`bX/+V0fHb!GXTB`nOhfu??4fL3^9IG\\!,Z3f9f/5M%5*u$Qo`J$jE\':tj94I9e?V\\%JIF*ua;:t,$"Zq/[*=?$jN6^9c\\[+WkK0.uIIrOj<?@8?R:Jb)%/QM.-PYhCH?N*qak+*/(:;WRpa3X,*P[JfTh?nF:YT(U:<<m6kmH<l:T@J.qR1[$bH^lb(lSJk#>CID\\jIi.:B!mojni:uX[tc`+W)+Sg0>\\ta>P(\\,I\'Dn$!$*iDY?=W(7!V)2!FV%PXn9[A1[\'F(_FCL[0pl[DcSpT0VGk]j<d[_*U(GfJNcJ"k9$(@M6E<eR,ufe@KoRRD3fRUboDC!\\W"RcmgnGR!3R.u:>DPG]1+S"V:G##qLLYc2aPkf2>bX2b?PJEmI`c`b=;H&>?a1ecY5~>'




"make form f1040ez from xml input"

import string

def main(xmlinput, formname, fileprefix):
    # parse the input
    from reportlab.lib.rparsexml import parsexml
    parsed = parsexml(xmlinput)
    tags = parsed[2]
    firsttag = tags[0]
    while isinstance(firsttag,str):
        del tags[0]
        firsttag = tags[0]
    attributes = firsttag[1]
    # now generate the rml from the attributes
    import preppy
    template = preppy.getModule("%sTemplate" % formname)
    rmlfilename = fileprefix + ".rml"
    rmlfile = open(rmlfilename, "w")
    template.run(attributes, outputfile=rmlfile)
    rmlfile.close()
    print("wrote", rmlfilename)
    rml = open(rmlfilename, "r").read()
    import string
    pdffilename = fileprefix + ".pdf"
    rml = string.replace(rml, '"test.pdf"', '"%s"' % pdffilename)
    # now process the rml into pdf
    from rlextra.rml2pdf import rml2pdf
    rml2pdf.go(rml)
    print("wrote", pdffilename)

if __name__=="__main__":
    import sys
    args = sys.argv[1:]
    if len(args)==2:
        xmlfilename = args[1]
        formname = args[0]
    else:
        print("usage: makeform.py formname InputFileName.xml")
        print("defaulting to f1040 f1040_example.xml")
        formname = "f1040"
        xmlfilename = "f1040_example.xml"
    xmlinput = open(xmlfilename).read()
    fileprefix = string.split(xmlfilename, ".")[0]
    main(xmlinput, formname, fileprefix)

    




"make preppy for a form, with matching xml example"

formname = "f1040"
formdatafile = "f1040fields.py"
dtdout = "f1040.dtd"
xmlout = "f1040_example.xml"

import string

def emitString(name, example, dtdfile, xmlfile):
    dtdfile.write("""
	%s CDATA #REQUIRED""" % name)
    xmlfile.write("""
	%s="%s" """ % (name, example))

count = 1000.99
def emit(name, alignment, description, fieldtype, dtdfile, xmlfile):
    global count
    count = count+1
    example = str(name)[:15]
    if fieldtype=="/Btn":
        example = "X"
    if not description:
        if alignment==2:
            return emitString(name, count, dtdfile, xmlfile)
        else:
            return emitString(name, example, dtdfile, xmlfile)
    elif description=="phone":
        example = "111 222 %s" % int(count)
    elif description=="PIN":
        example = int(count)
    elif description=="bigssn":
        example = "999 22 %s" % int(count)
    elif description=="ssn":
        example = "333 44 %s" % int(count)
    elif description=="account":
        example = "9999%s" % int(count)
    else:
        raise ValueError("unknown description: "+repr((description,name)))
    return emitString(name, example, dtdfile, xmlfile)

def gendtd(formname=formname, formdatafile=formdatafile, dtdout=dtdout, xmlout=xmlout):
    dtdfile = open(dtdout, "w")
    xmlfile = open(xmlout, "w")
    #formname = string.split(dtdout, ".")[0]
    xmlfile.write("""<!DOCTYPE %s SYSTEM "%s">\n\n<%s\n""" % (formname, dtdout, formname))
    dtdfile.write("""\n<!ELEMENT %s EMPTY>\n\n<!ATTLIST %s\n""" % (formname, formname))
    formdata = eval( open(formdatafile).read() )
    starting = 1
    count = 0
    seen = {}
    for page in formdata:
        for descriptor in page:
            name = descriptor["T"]
            if name in seen:
                # don't do it again
                continue
            seen[name] = name
            # ignore names that begin "c" or "f"
            if name[0] in "cf":
                continue
            fieldtype = descriptor["FT"]
            alignment = descriptor.get("Q", 0)
            description = descriptor.get("TU", None)
            [xmin,ymin,xmax,ymax] = descriptor["Rect"]
            dummy = emit(name, alignment, description, fieldtype, dtdfile, xmlfile)
        count = count+1
    dtdfile.write("\n>\n\n")
    dtdfile.close()
    xmlfile.write("\n /> <!-- end of %s -->" % formname)
    xmlfile.close()
    print("wrote", dtdout, "and", xmlout)
    return ""

if __name__=="__main__":
    import sys
    args = sys.argv[1:]
    if not args:
        print("usage: makedtd.py formname")
        print("defaulting to formname='f1040'")
        formname = "f1040"
    else:
        formname = args[0]
    formdatafile = "%sfields.py" % formname
    storagefile = "%s.storage" % formname
    dtdout = "%s.dtd" % formname
    xmlout = "%s_example.xml" % formname
    print(gendtd(formname=formname, formdatafile=formdatafile, dtdout=dtdout, xmlout=xmlout))


"""generate the rml source for rml userguide using 2 preppy passes"""
import sys
# this should be integrated into rml2pdf somehow
SHOW_PAGE_NO = 0

f = "rml_user_guide_generated.rml"

def gen(filename="test_newconstructs.rml"):
    print("using file", filename)
    # testing hack to try new paragraphs
    if 1:
        print("abusing paragraphs :c)")
        from rlextra.radxml import para
    print("running rml2pdf")
    from rlextra.rml2pdf import rml2pdf
    import os
    dtddir = os.path.abspath("..")
    from time import time
    now = time()
    rml2pdf.go(open(filename).read(), dtdDir=dtddir, verbose=1, ignoreDefaults=0)
    print("elapsed", time()-now)
    #print rml2pdf.__file__
    print("pdf generation complete")

profiling = 0
if __name__=="__main__":
    if profiling:
        import profile
        print("profiling")
        profile.run("gen()")
    else:
        #gen2(showPageNo=SHOW_PAGE_NO)
        gen()



"""
This is a collection of utilities used by the ``svglib`` code module.
"""

import re
from math import acos, ceil, copysign, cos, degrees, fabs, hypot, radians, sin, sqrt

from reportlab.graphics.shapes import mmult, rotate, translate, transformPoint


def split_floats(op, min_num, value):
    """Split `value`, a list of numbers as a string, to a list of float numbers.

    Also optionally insert a `l` or `L` operation depending on the operation
    and the length of values.
    Example: with op='m' and value='10,20 30,40,' the returned value will be
             ['m', [10.0, 20.0], 'l', [30.0, 40.0]]
    """
    floats = [float(seq) for seq in re.findall('(-?\d*\.?\d*(?:e[+-]\d+)?)', value) if seq]
    res = []
    for i in range(0, len(floats), min_num):
        if i > 0 and op in {'m', 'M'}:
            op = 'l' if op == 'm' else 'L'
        res.extend([op, floats[i:i + min_num]])
    return res


def normalise_svg_path(attr):
    """Normalise SVG path.

    This basically introduces operator codes for multi-argument
    parameters. Also, it fixes sequences of consecutive M or m
    operators to MLLL... and mlll... operators. It adds an empty
    list as argument for Z and z only in order to make the resul-
    ting list easier to iterate over.

    E.g. "M 10 20, M 20 20, L 30 40, 40 40, Z"
      -> ['M', [10, 20], 'L', [20, 20], 'L', [30, 40], 'L', [40, 40], 'Z', []]
    """

    # operator codes mapped to the minimum number of expected arguments
    ops = {
        'A': 7, 'a': 7,
        'Q': 4, 'q': 4, 'T': 2, 't': 2, 'S': 4, 's': 4,
        'M': 2, 'L': 2, 'm': 2, 'l': 2, 'H': 1, 'V': 1,
        'h': 1, 'v': 1, 'C': 6, 'c': 6, 'Z': 0, 'z': 0,
    }
    op_keys = ops.keys()

    # do some preprocessing
    result = []
    groups = re.split('([achlmqstvz])', attr.strip(), flags=re.I)
    op = None
    for item in groups:
        if item.strip() == '':
            continue
        if item in op_keys:
            # fix sequences of M to one M plus a sequence of L operators,
            # same for m and l.
            if item == 'M' and item == op:
                op = 'L'
            elif item == 'm' and item == op:
                op = 'l'
            else:
                op = item
            if ops[op] == 0:  # Z, z
                result.extend([op, []])
        else:
            result.extend(split_floats(op, ops[op], item))
            op = result[-2]  # Remember last op

    return result


def convert_quadratic_to_cubic_path(q0, q1, q2):
    """
    Convert a quadratic Bezier curve through q0, q1, q2 to a cubic one.
    """
    c0 = q0
    c1 = (q0[0] + 2. / 3 * (q1[0] - q0[0]), q0[1] + 2. / 3 * (q1[1] - q0[1]))
    c2 = (c1[0] + 1. / 3 * (q2[0] - q0[0]), c1[1] + 1. / 3 * (q2[1] - q0[1]))
    c3 = q2
    return c0, c1, c2, c3


# ***********************************************
# Helper functions for elliptical arc conversion.
# ***********************************************

def vector_angle(u, v):
    d = hypot(*u) * hypot(*v)
    c = (u[0] * v[0] + u[1] * v[1]) / d
    if c < -1:
        c = -1
    elif c > 1:
        c = 1
    s = u[0] * v[1] - u[1] * v[0]
    return degrees(copysign(acos(c), s))


def end_point_to_center_parameters(x1, y1, x2, y2, fA, fS, rx, ry, phi=0):
    '''
    See http://www.w3.org/TR/SVG/implnote.html#ArcImplementationNotes F.6.5
    note that we reduce phi to zero outside this routine
    '''
    rx = fabs(rx)
    ry = fabs(ry)

    # step 1
    if phi:
        phi_rad = radians(phi)
        sin_phi = sin(phi_rad)
        cos_phi = cos(phi_rad)
        tx = 0.5 * (x1 - x2)
        ty = 0.5 * (y1 - y2)
        x1d = cos_phi * tx - sin_phi * ty
        y1d = sin_phi * tx + cos_phi * ty
    else:
        x1d = 0.5 * (x1 - x2)
        y1d = 0.5 * (y1 - y2)

    # step 2
    # we need to calculate
    # (rx*rx*ry*ry-rx*rx*y1d*y1d-ry*ry*x1d*x1d)
    # -----------------------------------------
    #     (rx*rx*y1d*y1d+ry*ry*x1d*x1d)
    #
    # that is equivalent to
    #
    #          rx*rx*ry*ry
    # = -----------------------------  -    1
    #   (rx*rx*y1d*y1d+ry*ry*x1d*x1d)
    #
    #              1
    # = -------------------------------- - 1
    #   x1d*x1d/(rx*rx) + y1d*y1d/(ry*ry)
    #
    # = 1/r - 1
    #
    # it turns out r is what they recommend checking
    # for the negative radicand case
    r = x1d * x1d / (rx * rx) + y1d * y1d / (ry * ry)
    if r > 1:
        rr = sqrt(r)
        rx *= rr
        ry *= rr
        r = x1d * x1d / (rx * rx) + y1d * y1d / (ry * ry)
    r = 1 / r - 1
    if -1e-10 < r < 0:
        r = 0
    r = sqrt(r)
    if fA == fS:
        r = -r
    cxd = (r * rx * y1d) / ry
    cyd = -(r * ry * x1d) / rx

    # step 3
    if phi:
        cx = cos_phi * cxd - sin_phi * cyd + 0.5 * (x1 + x2)
        cy = sin_phi * cxd + cos_phi * cyd + 0.5 * (y1 + y2)
    else:
        cx = cxd + 0.5 * (x1 + x2)
        cy = cyd + 0.5 * (y1 + y2)

    # step 4
    theta1 = vector_angle((1, 0), ((x1d - cxd) / rx, (y1d - cyd) / ry))
    dtheta = vector_angle(
        ((x1d - cxd) / rx, (y1d - cyd) / ry),
        ((-x1d - cxd) / rx, (-y1d - cyd) / ry)
    ) % 360
    if fS == 0 and dtheta > 0:
        dtheta -= 360
    elif fS == 1 and dtheta < 0:
        dtheta += 360
    return cx, cy, rx, ry, -theta1, -dtheta


def bezier_arc_from_centre(cx, cy, rx, ry, start_ang=0, extent=90):
    if abs(extent) <= 90:
        nfrag = 1
        frag_angle = float(extent)
    else:
        nfrag = int(ceil(abs(extent) / 90.))
        frag_angle = float(extent) / nfrag

    frag_rad = radians(frag_angle)
    half_rad = frag_rad * 0.5
    kappa = abs(4. / 3. * (1. - cos(half_rad)) / sin(half_rad))

    if frag_angle < 0:
        kappa = -kappa

    point_list = []
    theta1 = radians(start_ang)
    start_rad = theta1 + frag_rad

    c1 = cos(theta1)
    s1 = sin(theta1)
    for i in xrange(nfrag):
        c0 = c1
        s0 = s1
        theta1 = start_rad + i * frag_rad
        c1 = cos(theta1)
        s1 = sin(theta1)
        point_list.append((cx + rx * c0,
                          cy - ry * s0,
                          cx + rx * (c0 - kappa * s0),
                          cy - ry * (s0 + kappa * c0),
                          cx + rx * (c1 + kappa * s1),
                          cy - ry * (s1 - kappa * c1),
                          cx + rx * c1,
                          cy - ry * s1))
    return point_list


def bezier_arc_from_end_points(x1, y1, rx, ry, phi, fA, fS, x2, y2):
    if phi:
        # Our box bezier arcs can't handle rotations directly
        # move to a well known point, eliminate phi and transform the other point
        mx = mmult(rotate(-phi), translate(-x1, -y1))
        tx2, ty2 = transformPoint(mx, (x2, y2))
        # Convert to box form in unrotated coords
        cx, cy, rx, ry, start_ang, extent = end_point_to_center_parameters(
            0, 0, tx2, ty2, fA, fS, rx, ry
        )
        bp = bezier_arc_from_centre(cx, cy, rx, ry, start_ang, extent)
        # Re-rotate by the desired angle and add back the translation
        mx = mmult(translate(x1, y1), rotate(phi))
        res = []
        for x1, y1, x2, y2, x3, y3, x4, y4 in bp:
            res.append(
                transformPoint(mx, (x1, y1)) + transformPoint(mx, (x2, y2)) +
                transformPoint(mx, (x3, y3)) + transformPoint(mx, (x4, y4))
            )
        return res
    else:
        cx, cy, rx, ry, start_ang, extent = end_point_to_center_parameters(
            x1, y1, x2, y2, fA, fS, rx, ry
        )
        return bezier_arc_from_centre(cx, cy, rx, ry, start_ang, extent)



__code__='Gas1]D0+/q&DiU]a"_:I9aZ<"U-`SALYC^`9W-A(PVJkC7PPbhUa+U@.,%<<,"pp;:)m"\'\'P@3<J#%Rh5=8fT0D_Z[MFCcXYjZb\\GIu2-]>!J-J[s*<RQo3Ds!EA"_#sHM"X(T2@!c^?-U8"u#$t"\'!,2G]0EEc@#%/*`!O)a;#hT>D"1SL/!T5`k=hkO*LO`[GF+>ck*s%r<]E%^]i$X&I!e(>u&b$FkkCAnUjm7&P\\aKEF&UksB$bQ\\b->+GQO:=0-6+lt+2P4.AN7`!_0/usmEju/qkK?b>L@YS;Fb*esXGW[V4K:Zjl?FVj1\\p4TZ\'6B7@;*r</Xd-;!sW_T6NW8NS:>7)"ur>c1\'m!+rrQBR4(abS2W2Z,lu)2?\\=(A=6rJ/nfS7?_s\'"i/4F/]/]QWcCA&g,:G,,VT<olY3>;S)M-/,Ur/!odL.?`+!k<#9?Q2Cfc761>4W)ug&$GO($U[Mq^=(-TeeXu%RcMNg-1r`jF=;k>^[H/A5U+NsN,784mQXrS3o)0gN9A_DW^->A4\'(^FkpKpUGpuFiN.B[-N1Q-(ZE<\\e:=V@<OG;Lf#m7/p)Q$.d3Hc(7eZD61blo5>`P"W/<73nS8e*T01?*1I%qu)/"K:])$j\'\\o(VW47BQp\\@HXmHA`,g]DWfVu1<1lE&VKkY`*Iip)6+H(j,iqactC6IhMr;HVI_QNjplaL2*IA=T+WH-9XVl?dD;<\\bGFgnk9QhV!^/PrHdUUW<T\'-0nV:62TneII?_ltiE5h&r%I$="[!b;WL@XO=V"j4i=4DLKQp:Q"Qhb(E)TPlrff[.`gYpaR3RaXIZHiQh95jS!C]0E\\fJKY\'"pMisXBK=($K(#?Hnp&At_-ZjQO^K>d;Bb%MZK.*&,5nAlsI?a0i4aC4Q7l$U580?P]UGT,WoNK1\\Z\'r\')HXs!\\Z_F%LlA^#p^cE]E99M_Ucl:Aebt;PV30Q5WDi@"W\\!V<?pnBgD\'[o"DEnU(8a^&@BX%K\'1</2P"7oW#SX\'G/m+H?c.6FI`?Cd_Heo2=q?@UHCaWh=R8V/6*LU9>Y!4C@4``OY-*0KZgI&P_:M4>UOS~>'
